#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
SchoolTool Niepa initialization.
"""

from zope.annotation.interfaces import IAnnotations
from zope.container.interfaces import INameChooser
from zope.interface import implements
from zope.schema import List, TextLine, Int, Float, Choice, Bool, Date
from zope.schema.interfaces import IDate

from schooltool.app.app import InitBase, StartUpBase, ApplicationPreferences
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.interfaces import IApplicationPreferences
from schooltool.app.utils import vocabulary
from schooltool.basicperson.demographics import TextFieldDescription
from schooltool.basicperson.demographics import BoolFieldDescription
from schooltool.basicperson.demographics import EnumFieldDescription
from schooltool.basicperson.interfaces import IDemographicsFields
from schooltool.group.interfaces import IGroupContainer
from schooltool.resource.interfaces import IResourceDemographicsFields
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.schoolyear.interfaces import ISchoolYearContainer
from schooltool.term.interfaces import ITerm

from schooltool.gradebook.interfaces import IGradebookRoot
from schooltool.gradebook.activity import ReportWorksheet, ReportActivity
from schooltool.gradebook.activity import Worksheet
from schooltool.gradebook.browser.report_card import copyActivities
from schooltool.requirement.scoresystem import RangedValuesScoreSystem

from schooltool.niepa import NiepaMessage as _


APP_PREFERENCES_KEY = 'schooltool.niepa.app.ApplicationPreferences'
DEFAULT_NIEPA_TITLE = _('Niepa')
EXAMS_REPORTSHHET_TITLE = _('Exams')
EXAM_TEMPLATE_NAME = 'niepa_exams'


DISTANCE_DESCRIPTION = _("""How many kilometres is the school away from the
 LGA capital?""")
SCHOOL_TYPE_DESCRIPTION = _("""Does your school fall into any of these special
 categories?""")
SHARED_FACILITLES_DESCRIPTION = _("""Does the school share facilities /
 premises with any other school?""")
DEVELOPMENT_PLAN_DESCRIPTION = _("""Did the school prepare a School Development
 Plan in the last school year?""")
SMBC_DESCRIPTION = _("""Does the school have a School Based Management
 Committee, which has met at least once in the last school year?'""")
PARENT_FORM_DESCRIPTION = _("""Does the school have a Parent-Teacher
 Association / Parents Forum, which has met at least once in the last
 school year?""")


class INiepaApplicationPreferences(IApplicationPreferences):

    name = TextLine(title=_('School Name'), required=False)
    street = TextLine(title=_('Number and Street'), required=False)
    village = TextLine(title=_('Village'), required=False)
    town = TextLine(title=_('Town'), required=False)
    lga = TextLine(title=_('LGA'), required=False)
    state = TextLine(title=_('State'), required=False)
    telephone = TextLine(title=_('Telephone'), required=False)

    school_code = TextLine(title=_('School code'),
        required=False)
    location_coordinates = TextLine(title=_('Location coordinates'),
        required=False)
    year_established = TextLine(title=_('Year of establishment'),
        required=False)
    distance = Float(title=_('Distance (km)'), required=False,
        description=DISTANCE_DESCRIPTION)
    levels_offered = Choice(title=_("Levels of education offered"),
        required=False,
        vocabulary=vocabulary([
            ("junior_only", _("Junior secondary only")),
            ("junior_senior", _("Junior and senior secondary")),
            ]))
    school_type = Choice(title=_("Type of school"), required=False,
        description=SCHOOL_TYPE_DESCRIPTION,
        vocabulary=vocabulary([("private", _("Private")),
                               ("public", _("Public")),
                               ("private_vocational", _("Private Vocational")),
                               ("public_vocational", _("Public Vocational")),
                               ("mission", _("Mission")),
                               ("demonstration", _("Demonstration")),
                               ]))
    shifts = Choice(title=_("Shifts"), required=False,
        description=_('How many shifts does the school operate?'),
        vocabulary=vocabulary([("single", _("Single shift")),
                               ("double", _("Double shift")),
                               ]))
    shared_facilities = Bool(title=_("Shared facilities"), required=False,
        description=SHARED_FACILITLES_DESCRIPTION)
    students_boarding = Bool(title=_("Students: Boarding"), required=False,
        description=_('How many students board at the school premises?'))
    development_plan = Bool(title=_("School Development Plan"), required=False,
        description=DEVELOPMENT_PLAN_DESCRIPTION)
    sbmc = Bool(title=_("School Based Management Committee (SBMC)"),
        required=False,
        description=SMBC_DESCRIPTION)
    parent_forum = Bool(
        title=_("Parent-Teacher Association (PTA) / Parents Forum (PF)"),
        required=False,
        description=PARENT_FORM_DESCRIPTION)
    last_inspection = Date(title=_("Date of last inspection visit"),
        required=False,
        description=_('What was the date of the last inspection visit?'))
    inspection_authority = Choice(title=_("Authority of last inspection"),
        required=False,
        description=_('Which authority conducted the last inspection visit?'),
        vocabulary=vocabulary([("federal", _("Federal")),
                               ("state", _("State")),
                               ("lgea", _("LGEA")),
                               ]))

class NiepaApplicationPreferences(ApplicationPreferences):

    implements(INiepaApplicationPreferences)
    school_code = None
    location_coordinates = None

    def __init__(self, *args, **kw):
        super(NiepaApplicationPreferences, self).__init__(*args, **kw)
        self.title = DEFAULT_NIEPA_TITLE
        self.frontPageCalendar = False

        self.name = None
        self.street = None
        self.village = None
        self.town = None
        self.lga = None
        self.state = None
        self.telephone = None

        self.year_established = None
        self.distance = None
        self.levels_offered = None
        self.school_type = None
        self.shifts = None
        self.shared_facilities = None
        self.students_boarding = None
        self.development_plan = None
        self.sbmc = None
        self.parent_forum = None
        self.last_inspection = None
        self.inspection_authority = None


class NiepaAppInit(InitBase):

    after = ('schooltool.basicperson.demographics', )

    def __call__(self):
        create_demo_fields()
        create_resource_demo_fields()


def create_demo_fields():

    dfs = IDemographicsFields(ISchoolToolApplication(None))

    # student fields
    dfs['visually_impaired'] = BoolFieldDescription(
        'visually_impaired', _('Blind or visually impaired'),
        limit_keys=['students'])
    dfs['hearing_or_speech'] = BoolFieldDescription(
        'hearing_or_speech', _('Hearing or speech impaired'),
        limit_keys=['students'])
    dfs['physically_challenged'] = BoolFieldDescription(
        'physically_challenged', _('Physically Challenged'),
        limit_keys=['students'])
    dfs['mentally_challenged'] = BoolFieldDescription(
        'mentally_challenged', _('Mentally Challenged'),
        limit_keys=['students'])
    dfs['registered_jsce'] = BoolFieldDescription(
        'registered_jsce', _('Registered for JSCE'),
        limit_keys=['students'])
    dfs['participated_jsce'] = BoolFieldDescription(
        'participated_jsce', _('Participated in JSCE'),
        limit_keys=['students'])
    dfs['passed_jsce'] = BoolFieldDescription(
        'passed_jsce', _('Passed JSCE'),
        limit_keys=['students'])

    # teacher fields
    dfs['type_of_teacher'] = EnumFieldDescription(
        'type_of_teacher', _('Type of teacher'),
        limit_keys=['teachers'])
    dfs['type_of_teacher'].items = [
        u'Principal',
        u'Vice principal',
        u'Teacher']
    dfs['employer'] = EnumFieldDescription(
        'employer', _('Employer'),
        limit_keys=['teachers'])
    dfs['employer'].items = [
        u'Goverment',
        u'Other, e.g. volunteer, paid by the community, PTA teacher etc.']
    dfs['present'] = EnumFieldDescription(
        'present', _('Present'),
        limit_keys=['teachers'])
    dfs['present'].items = [
        u'Present or temporarily absent',
        u'Absent for more than 1 month - Maternity leave',
        u'Absent for more than 1 month - Sick leave',
        u'Absent for more than 1 month - Training',
        u'Absent for more than 1 month - Secondment',
        u'Absent for more than 1 month - Unauthorised']
    dfs['academic_qualification'] = EnumFieldDescription(
        'academic_qualification', _('Academic qualification'),
        limit_keys=['teachers'])
    dfs['academic_qualification'].items = [
        u'Below SSCE',
        u'SSCE/WASC',
        u'OND / Diploma',
        u'Degree / HND / Graduate']
    dfs['teaching_qualification'] = EnumFieldDescription(
        'teaching_qualification', _('Teaching qualification'),
        limit_keys=['teachers'])
    dfs['teaching_qualification'].items = [
        u'NCE',
        u'PGDE',
        u'B.Ed. or equivalent',
        u'M.Ed. or equivalent',
        u'Grade II or equivalent']
    dfs['subject_qualification'] = EnumFieldDescription(
        'subject_qualification', _('Subject qualification'),
        limit_keys=['teachers'])
    dfs['subject_qualification'].items = [
        u'English',
        u'Mathematics',
        u'Social studies',
        u'Basic science',
        u'Hausa/Igbo/Yoruba'
        u'Other']


def create_demo_fields_again():

    dfs = IDemographicsFields(ISchoolToolApplication(None))

    # teacher fields
    dfs['year_of_first_appointment'] = TextFieldDescription(
        'year_of_first_appointment', _('Year of first appointment'),
        limit_keys=['teachers'])
    dfs['year_of_present_appointment'] = TextFieldDescription(
        'year_of_present_appointment', _('Year of present appointment'),
        limit_keys=['teachers'])


def create_resource_demo_fields():

    dfs = IResourceDemographicsFields(ISchoolToolApplication(None))

    # location fields
    dfs['block'] = TextFieldDescription(
        'block', _('Block'),
        limit_keys=['location'])
    dfs['present_condition'] = EnumFieldDescription(
        'present_condition', _('Present condition'),
        limit_keys=['location'])
    dfs['present_condition'].items = [
        u'Good',
        u'Needs minor repairs',
        u'Needs major repairs',
        u'Under construction',
        u'Unusable']
    dfs['floor_material'] = EnumFieldDescription(
        'floor_material', _('Floor material'),
        limit_keys=['location'])
    dfs['floor_material'].items = [
        u'Mud/Earth',
        u'Concrete',
        u'Wood/Plank',
        u'Tile']
    dfs['wall_material'] = EnumFieldDescription(
        'wall_material', _('Wall material'),
        limit_keys=['location'])
    dfs['wall_material'].items = [
        u'Mud',
        u'Cement/Concrete',
        u'Wood/Bamboo',
        u'Burnt bricks',
        u'Iron sheets',
        u'Stone']
    dfs['roof_material'] = EnumFieldDescription(
        'roof_material', _('Roof material'),
        limit_keys=['location'])
    dfs['roof_material'].items = [
        u'Mud',
        u'Cement/Concrete',
        u'Wood/Bamboo',
        u'Ceramic tiles',
        u'Iron sheets']
    dfs['seating'] = BoolFieldDescription(
        'seating', _('Seating'),
        limit_keys=['location'])
    dfs['good_blackboard'] = BoolFieldDescription(
        'good_blackboard', _('Good blackboard'),
        limit_keys=['location'])


def create_resource_demo_fields_again():

    dfs = IResourceDemographicsFields(ISchoolToolApplication(None))

    # location fields
    dfs['year_of_construction'] = TextFieldDescription(
        'year_of_construction', _('Year of construction'),
        limit_keys=['location'])
    dfs['length_in_meters'] = TextFieldDescription(
        'length_in_meters', _('Length in meters'),
        limit_keys=['location'])
    dfs['width_in_meters'] = TextFieldDescription(
        'width_in_meters', _('Width in meters'),
        limit_keys=['location'])


class NiepaAppStartup(StartUpBase):

    def __call__(self):
        root = IGradebookRoot(ISchoolToolApplication(None))
        if EXAM_TEMPLATE_NAME not in root.templates:
            ss15 = RangedValuesScoreSystem(u'generated',  min=0, max=15)
            ss10 = RangedValuesScoreSystem(u'generated',  min=0, max=10)
            ss60 = RangedValuesScoreSystem(u'generated',  min=0, max=60)
            worksheet = ReportWorksheet(EXAMS_REPORTSHHET_TITLE)
            root.templates[EXAM_TEMPLATE_NAME] = worksheet
            worksheet['1'] = ReportActivity('1stCo', 'exam', ss15)
            worksheet['2'] = ReportActivity('2ndCo', 'exam', ss15)
            worksheet['3'] = ReportActivity('3rdCo', 'exam', ss10)
            worksheet['4'] = ReportActivity('Third', 'exam', ss60)

        dfs = IDemographicsFields(self.app)
        if 'visually_impaired' not in dfs:
            create_demo_fields()
            create_resource_demo_fields()
        if 'year_of_first_appointment' not in dfs:
            create_demo_fields_again()
            create_resource_demo_fields_again()


def getApplicationPreferences(app):
    key = APP_PREFERENCES_KEY
    annotations = IAnnotations(app)
    try:
        return annotations[key]
    except KeyError:
        annotations[key] = NiepaApplicationPreferences()
        annotations[key].__parent__ = app
        return annotations[key]


def handleTermAdded(event):
    """Make sure the required reportsheet is deployed to newly added terms."""

    if not ITerm.providedBy(event.object):
        return
    term = event.object
    schoolyear = ISchoolYear(term)

    root = IGradebookRoot(ISchoolToolApplication(None))
    worksheet = root.templates[EXAM_TEMPLATE_NAME]
    deployedWorksheet = Worksheet(worksheet.title)

    chooser = INameChooser(root.deployed)
    deployedKey = '%s_%s' % (schoolyear.__name__, term.__name__)
    name = chooser.chooseName(deployedKey, deployedWorksheet)
    root.deployed[name] = deployedWorksheet
    copyActivities(worksheet, deployedWorksheet)


def activeSchoolYear(context=None):
    app = ISchoolToolApplication(context)
    syc = ISchoolYearContainer(app)
    return syc.getActiveSchoolYear()


def activeGroupContainer(context=None):
    active_schoolyear = activeSchoolYear(context)
    if active_schoolyear is None:
        return {}
    return IGroupContainer(active_schoolyear)

