#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
SchoolTool Niepa census form view.
"""

from decimal import Decimal
from reportlab.platypus.flowables import Flowable

from z3c.form.field import Fields

from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.publisher.browser import BrowserView
from zope.schema.interfaces import IChoice, IBool
from zope.security.proxy import removeSecurityProxy
from zope.traversing.browser.absoluteurl import absoluteURL

from schooltool.app.browser.report import ReportPDFView
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.basicperson.interfaces import IDemographicsFields
from schooltool.basicperson.interfaces import IDemographics
from schooltool.course.interfaces import IInstructor, ISectionContainer
from schooltool.group.interfaces import IGroupContainer
from schooltool.resource.interfaces import IResourceDemographicsFields
from schooltool.resource.interfaces import IResourceDemographics
from schooltool.resource.interfaces import ILocation
from schooltool.schoolyear.interfaces import ISchoolYearContainer
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.term.interfaces import ITerm

from schooltool.gradebook.interfaces import IActivities
from schooltool.requirement.interfaces import IEvaluations
from schooltool.requirement.scoresystem import UNSCORED

from schooltool.niepa.app.app import INiepaApplicationPreferences

from schooltool.niepa import NiepaMessage as _


class CensusFormPDFView(ReportPDFView):
    """A view for the school's census form pdf"""

    template=ViewPageTemplateFile('templates/census_form_pdf.pt')

    def __call__(self):
        self.year = removeSecurityProxy(self.context)
        self.app = ISchoolToolApplication(None)
        self.persons = self.app['persons']
        self.resources = self.app['resources']

        self.student_ids = set()
        self.teacher_ids = set()
        for term in self.year.values():
            for section in ISectionContainer(term).values():
                for student in section.members:
                    self.student_ids.add(student.username)
                for teacher in section.instructors:
                    self.teacher_ids.add(teacher.username)
        self.student_ids = list(self.student_ids)
        self.teacher_ids = list(self.teacher_ids)

        return super(CensusFormPDFView, self).__call__()

    def pages(self):
        result = {}
        result['page1'] = self.getPage1()
        result['enrollment'] = self.getEnrollment()
        result['staff'] = self.getStaff()
        result['classrooms'] = self.getClassrooms()
        return result

    def getPage1(self):
        result = {}
        prefs = INiepaApplicationPreferences(self.app)

        result['school_code'] = prefs.school_code
        result['year_title'] = self.year.title

        fields = Fields(INiepaApplicationPreferences).select('name', 'street',
            'village', 'town', 'lga', 'state', 'telephone')
        rows = []
        for field in fields.values():
            row = {
                'title': field.field.title,
                'value': getattr(prefs, field.__name__),
                }
            rows.append(row)
        result['identification_rows'] = rows

        fields = Fields(INiepaApplicationPreferences).select('year_established',
            'location_coordinates', 'distance', 'levels_offered', 'school_type',
            'shifts', 'shared_facilities', 'development_plan', 'sbmc',
            'parent_forum', 'last_inspection', 'inspection_authority')
        prefs = INiepaApplicationPreferences(self.app)
        rows = []
        for field in fields.values():
            value = getattr(prefs, field.__name__)
            if value is None:
                value = ''
            if value != '':
                if IChoice.providedBy(field.field):
                    value = field.field.vocabulary.getTerm(value).title
                elif IBool.providedBy(field.field):
                    value = {True: 'Yes', False: 'No'}[value]
            row = {
                'title': field.field.title,
                'value': value,
                }
            rows.append(row)
        result['characteristics_rows'] = rows

        return result

    def getEnrollment(self):
        result = {}
        dfs = IDemographicsFields(ISchoolToolApplication(None))
        disabilities_names = ['visually_impaired', 'hearing_or_speech',
            'physically_challenged', 'mentally_challenged']
        jsce_names = ['registered_jsce', 'participated_jsce', 'passed_jsce']

        disabilities, jsce = {}, {}
        for name in disabilities_names:
            field = dfs[name]
            disabilities[name] = {
                'challenge': field.title,
                'male': 0,
                'female': 0,
                }
        for name in jsce_names:
            field = dfs[name]
            jsce[name] = {
                'setting': field.title,
                'male': 0,
                'female': 0,
                'total': 0,
                }

        for student_id in self.student_ids:
            student = self.persons[student_id]
            gender = student.gender
            if gender is None:
                continue
            demos = removeSecurityProxy(IDemographics(student))

            for name in disabilities_names:
                field = dfs[name]
                value = demos.get(name, '')
                if value != '':
                    disabilities[name][gender] += 1
            for name in jsce_names:
                field = dfs[name]
                value = demos.get(name, '')
                if value != '':
                    jsce[name][gender] += 1
                    jsce[name]['total'] += 1

        result['disabilities_rows'] = [disabilities[name]
                                       for name in disabilities_names]
        result['jsce_rows'] = [jsce[name] for name in jsce_names]

        return result

    def getStaff(self):
        result = []
        dfs = IDemographicsFields(ISchoolToolApplication(None))
        names = ['type_of_teacher', 'employer', 'present',
            'year_of_first_appointment', 'year_of_present_appointment',
            'academic_qualification', 'teaching_qualification',
            'subject_qualification']

        teachers = [self.persons[teacher_id] for teacher_id in self.teacher_ids]
        teachers_sorter = [(teacher.title, teacher) for teacher in teachers]
        teachers = [teacher for name, teacher in sorted(teachers_sorter)]

        rows, number = [], 1
        for teacher in teachers:
            gender = teacher.gender
            if gender is None:
                gender = ''
            if gender == 'male':
                gender = 'M'
            elif gender == 'female':
                gender = 'F'

            if teacher.birth_date is None:
                year_of_birth = ''
            else:
                year_of_birth = teacher.birth_date.year

            row = {
                'number': number,
                'name': teacher.title,
                'gender': gender,
                'year_of_birth': year_of_birth,
                }

            demos = removeSecurityProxy(IDemographics(teacher))
            for name in names:
                field = dfs[name]
                row[name] = demos.get(name, '')
                if row[name] and 'items' in dir(field):
                    row[name] = field.items.index(row[name]) + 1

            rows.append(row)
            number += 1

        per_page = 40
        while rows:
            page = {
                'rows': rows[:per_page],
                }
            result.append(page)
            rows = rows[per_page:]

        return result

    def getClassrooms(self):
        result = []
        dfs = IResourceDemographicsFields(ISchoolToolApplication(None))
        names = ['block', 'present_condition', 'floor_material',
            'wall_material', 'roof_material', 'seating', 'good_blackboard',
            'year_of_construction', 'length_in_meters', 'width_in_meters']

        rows, number = [], 1
        for room in self.resources.values():
            if not ILocation.providedBy(room):
                continue
            row = {
                'number': number,
                }

            demos = removeSecurityProxy(IResourceDemographics(room))
            for name in names:
                field = dfs[name]
                row[name] = demos.get(name, '')
                if row[name] != '':
                    if 'EnumFieldDescription' in str(field):
                        row[name] = field.items.index(row[name]) + 1
                    elif 'BoolFieldDescription' in str(field):
                        row[name] = {True: 1, False: 2}[row[name]]

            rows.append(row)
            number += 1

        per_page = 40
        while rows:
            page = {
                'rows': rows[:per_page],
                }
            result.append(page)
            rows = rows[per_page:]

        return result

