#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
SchoolTool Niepa report card view.
"""

from decimal import Decimal
from reportlab.platypus.flowables import Flowable

from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.publisher.browser import BrowserView
from zope.security.proxy import removeSecurityProxy
from zope.traversing.browser.absoluteurl import absoluteURL

from schooltool.app.browser.report import ReportPDFView
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.course.interfaces import IInstructor, ISectionContainer
from schooltool.group.interfaces import IGroupContainer
from schooltool.schoolyear.interfaces import ISchoolYearContainer
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.term.interfaces import ITerm

from schooltool.gradebook.interfaces import IActivities
from schooltool.requirement.interfaces import IEvaluations
from schooltool.requirement.scoresystem import UNSCORED

from schooltool.niepa import NiepaMessage as _


class RotatedText(Flowable):

    def __init__(self, text):
        self.text = text

    def draw(self):
        self.canv.saveState()
        canvas = self.canv
        canvas.rotate(90)
        canvas.drawString(5, -10, self.text)
        self.canv.restoreState()

    def wrap(self,aW,aH):
        canv = self.canv
        return canv._leading, canv.stringWidth(self.text)


class ReportCardPDFView(ReportPDFView):
    """A view for the term's report card pdf"""

    template=ViewPageTemplateFile('templates/report_card_pdf.pt')
    per_page = 26

    def __call__(self):
        self.term = removeSecurityProxy(self.context)
        self.year = ISchoolYear(self.term)
        terms = tuple(self.year.values())
        this_index = terms.index(self.term)

        courses = set()
        for term in terms:
            for section in ISectionContainer(term).values():
                courses.add(tuple(section.courses))
        courses = list(courses)

        course_accums = {}
        for course in courses:
            scores = course_accums[course] = []
            for index in range(this_index + 1):
                term = terms[index]
                for section in ISectionContainer(term).values():
                    if tuple(section.courses) == course:
                        break
                else:
                    section = None
                scores.append(self.getSectionScores(section, term))

        self.data = course_accums
        return super(ReportCardPDFView, self).__call__()

    def getSectionScores(self, section, term):
        results = {}
        activities = IActivities(section)
        key = '%s_%s' % (self.year.__name__, term.__name__)
        if key not in activities:
            return results
        worksheet = activities[key]

        for student in section.members:
            scores = results[student] = []
            evaluations = IEvaluations(student)
            for activity in worksheet.values():
                ev = evaluations.get(activity, None)
                if ev is not None and ev.value is not UNSCORED:
                    value = ev.value
                else:
                    value = Decimal('0')
                scores.append(value)
        return results

    def pages(self):
        results = []
        courses_tuple = [(', '.join([c.title for c in course]), course)
                         for course in self.data]

        for title, course in sorted(courses_tuple):
            scores = self.data[course]
            students = [(student.title, student, scores[-1][student])
                        for student in scores[-1]]

            rows, index = [], 0
            for student_title, student, student_scores in sorted(students):
                ca1 = student_scores[0]
                ca2 = student_scores[1]
                ca3 = student_scores[2]
                ca_total = ca1 + ca2 + ca3
                exam = student_scores[3]
                total = ca_total + exam

                if len(scores) == 1:
                    before = ''
                    cumulative = total
                else:
                    before = Decimal('0')
                    for prev_index in range(len(scores) - 1):
                        for stu, stu_scores in scores[prev_index].items():
                            if stu == student:
                                before += sum(stu_scores)
                    if len(scores) == 2:
                        cumulative = int(round((total + before) / 2))
                    else:
                        cumulative = int(round((total + before) / 3))
                        before = int(round(before / 2))

                index += 1
                grade, remarks = '', ''
                row = {
                    'index': index,
                    'name': student_title,
                    'ca_test1': ca1,
                    'ca_test2': ca2,
                    'ca_test3': ca3,
                    'ca_total': ca_total,
                    'exam': exam,
                    'total': total,
                    'before': before,
                    'cumulative': cumulative,
                    'grade': grade,
                    'remarks': remarks,
                    }
                rows.append(row)

            class_average = Decimal('0')
            for row in rows:
                class_average += Decimal(row['cumulative'])
            if rows:
                class_average = int(round(class_average / len(rows)))
            for row in rows:
                row['class_average'] = class_average

            row_tuples = [(row['cumulative'], row) for row in rows]
            row_tuples = reversed(sorted(row_tuples))
            for index, (cumulative, row) in enumerate(row_tuples):
                row['position'] = index + 1

            while rows:
                page_rows = rows[:self.per_page]
                result = {
                    'course': title,
                    'term': self.term.title,
                    'rows': page_rows,
                    'heights': '0.9cm,3.5cm' + ',0.7cm' * (len(page_rows)),
                    }
                results.append(result)
                rows = rows[self.per_page:]

        return results

    def widths(self):
        return '0.8cm,5cm' + ',0.8cm' * 11 + ',4cm'

