#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Niepa person browser views.
"""

from zope.app.pagetemplate import ViewPageTemplateFile
from zope.component import adapts, getUtility, queryMultiAdapter
from zope.container.interfaces import INameChooser
from zope.event import notify
from zope.interface import implements, Interface, implementsOnly, alsoProvides
from zope.i18n import translate
from zope.lifecycleevent import ObjectCreatedEvent
from zope.schema import Choice, ValidationError
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from zope.security.proxy import removeSecurityProxy
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.viewlet.viewlet import ViewletBase
from z3c.form import form, field, button, subform, validator, widget
from z3c.form.interfaces import DISPLAY_MODE, HIDDEN_MODE, IActionHandler

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.basicperson.browser.person import IPersonAddForm
from schooltool.basicperson.browser.person import PersonAddFormBase
from schooltool.basicperson.browser.person import MultiplePersonAddView
from schooltool.basicperson.interfaces import IBasicPerson
from schooltool.basicperson.interfaces import IDemographicsFields
from schooltool.person.browser.person import PersonContainerView
from schooltool.skin.containers import ContainerDeleteView
from schooltool.skin.skin import ActionMenuViewlet
from schooltool.skin.skin import NavigationViewlet

from schooltool.niepa.app.app import INiepaApplicationPreferences
from schooltool.niepa.app.app import activeGroupContainer, activeSchoolYear
from schooltool.niepa.person.browser.table import FilterByGroupTableFormatter

from schooltool.niepa import NiepaMessage as _


###############  Persons cntainer views (by group) ######################
class PersonsViewBase(PersonContainerView):

    template = ViewPageTemplateFile('templates/person_container.pt')
    delete_template = ViewPageTemplateFile('templates/person_delete.pt')

    group_id = None

    def __init__(self, context, request):
        self.context = context
        self.request = request
        formatter = FilterByGroupTableFormatter(self.context, self.request)
        formatter.group = activeGroupContainer().get(self.group_id)
        self.table = formatter


class TeachersView(PersonsViewBase):

    index_title = _('Teacher index')
    group_id = 'teachers'


class StudentsView(PersonsViewBase):

    index_title = _('Student index')
    group_id = 'students'


class AdministratorsView(PersonsViewBase):

    index_title = _('Administrator index')
    group_id = 'administrators'


class PersonContainerDeleteView(ContainerDeleteView):

    def nextURL(self):
        url = absoluteURL(self.container, self.request)
        if 'nextURL' in self.request:
            url += '/' + self.request['nextURL']
        return url


###########  Base class of all person views (uses self.group_id) ##############
class PersonFieldGenerator(object):

    def makeRows(self, fields, cols=1):
        rows = []
        while fields:
            rows.append(fields[:cols])
            fields = fields[cols:]
        return rows

    def makeFieldSet(self, fieldset_id, legend, fields, cols=1):
        result = {
            'id': fieldset_id,
            'legend': legend,
            }
        result['rows'] = self.makeRows(fields, cols)
        return result

    def fieldsets(self):
        result = []
        sources = [
            (self.base_id, self.base_legend, list(self.getBaseFields())),
            (self.demo_id, self.demo_legend, list(self.getDemoFields())),
            ]
        for fieldset_id, legend, fields in sources:
            result.append(self.makeFieldSet(fieldset_id, legend, fields, 2))
        return result

    def getGroup(self):
        groups = list(self.context.groups)
        gc = activeGroupContainer()
        for group in ['teachers', 'students', 'administrators']:
            if gc[group] in groups:
                return gc[group]

    def getDemoFields(self):
        fields = field.Fields()
        dfs = IDemographicsFields(ISchoolToolApplication(None))
        for field_desc in dfs.filter_key(self.group_id):
            fields += field_desc.makeField()
        return fields


###############  Person view (group determined by base class) #################
class PersonView(form.Form, PersonFieldGenerator):

    template = ViewPageTemplateFile('templates/person_view.pt')
    mode = DISPLAY_MODE
    id = 'person-view'
    fields_order = ['prefix', 'last_name', 'first_name', 'gender',
                    'birth_date']

    @property
    def label(self):
        return self.context.title

    def update(self):
        self.group_id = self.getGroup()

        self.fields = field.Fields(IBasicPerson)
        self.fields += self.getDemoFields()
        self.fields = self.fields.select(*self.fields_order)
        self.subforms = []
        super(PersonView, self).update()

    def __call__(self):
        self.update()
        return self.render()

    def updateWidgets(self):
        super(PersonView, self).updateWidgets()
        for widget in self.widgets:
            if not self.widgets[widget].value:
                self.widgets[widget].mode = HIDDEN_MODE


###############  Base classes of all person add/edit views ################
class PersonAddView(PersonAddFormBase, PersonFieldGenerator):

    id = 'person-form'
    template = ViewPageTemplateFile('templates/person_form.pt')

    def getBaseFields(self):
        return field.Fields(IPersonAddForm)

    def groupViewURL(self):
        return '%s/%s.html' % (absoluteURL(self.context, self.request),
                               self.group_id)

    def updateActions(self):
        super(PersonAddView, self).updateActions()
        self.actions['add'].addClass('button-ok')
        self.actions['cancel'].addClass('button-cancel')

    @button.buttonAndHandler(_('Add'))
    def handleAdd(self, action):
        super(PersonAddView, self).handleAdd.func(self, action)

    @button.buttonAndHandler(_('Cancel'))
    def handle_cancel_action(self, action):
        self.request.response.redirect(self.groupViewURL())

    def create(self, data):
        username = data.get('username')
        first_name = data.get('first_name')
        last_name = data.get('last_name')
        person = self._factory(username, first_name, last_name)
        data.pop('confirm')
        form.applyChanges(self, person, data)
        group = activeGroupContainer().get(self.group_id)
        if group is not None:
            person.groups.add(group)
        self._person = person
        return person

    def update(self):
        self.fields = field.Fields(IPersonAddForm)
        self.fields += self.getDemoFields()
        self.updateWidgets()
        self.updateActions()
        self.actions.execute()

    def updateWidgets(self):
        super(PersonAddView, self).updateWidgets()


class PersonEditView(form.EditForm, PersonFieldGenerator):

    id = 'person-form'
    template = ViewPageTemplateFile('templates/person_form.pt')

    def getBaseFields(self):
        return field.Fields(IBasicPerson)

    def update(self):
        self.fields = field.Fields(IBasicPerson)
        self.fields += self.getDemoFields()
        super(PersonEditView, self).update()

    @button.buttonAndHandler(_('Apply'))
    def handle_apply_action(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        self.applyChanges(data)
        url = absoluteURL(self.context, self.request)
        self.request.response.redirect(url)

    @button.buttonAndHandler(_('Cancel'))
    def handle_cancel_action(self, action):
        url = absoluteURL(self.context, self.request)
        self.request.response.redirect(url)

    @property
    def label(self):
        return _(u'Change information for ${fullname}',
                 mapping={'fullname': self.context.title})

    def updateActions(self):
        super(PersonEditView, self).updateActions()
        self.actions['apply'].addClass('button-ok')
        self.actions['cancel'].addClass('button-cancel')


###############  Teacher add/edit views ################
class TeacherFormBase(object):

    group_id = 'teachers'
    base_id = 'base-data'
    base_legend = _('Teacher identification')
    demo_id = 'demo-data'
    demo_legend = _('Teacher demographics')


class TeacherAddView(TeacherFormBase, PersonAddView):

    label = _('Add new teacher')


class TeacherEditView(TeacherFormBase, PersonEditView):

    label = _('Edit teacher')


###############  Student add/edit views ################
class StudentFormBase(object):

    group_id = 'students'
    base_id = 'base-data'
    base_legend = _('Student identification')
    demo_id = 'demo-data'
    demo_legend = _('Student demographics')


class StudentAddView(StudentFormBase, PersonAddView):

    label = _('Add new student')


class StudentEditView(StudentFormBase, PersonEditView):

    label = _('Edit student')


###############  Administrator add/edit views ################
class AdministratorFormBase(object):

    group_id = 'administrators'
    base_id = 'base-data'
    base_legend = _('Administrator identification')
    demo_id = 'demo-data'
    demo_legend = _('Administrator demographics')


class AdministratorAddView(AdministratorFormBase, PersonAddView):

    label = _('Add new administrator')


class AdministratorEditView(AdministratorFormBase, PersonEditView):

    label = _('Edit administrator')


###############  Viewlets ################
class EditActionMenuViewlet(ActionMenuViewlet):

    @property
    def link(self):
        gc = activeGroupContainer()
        for group in ['teachers', 'students', 'administrators']:
            if self.context in gc[group].members:
                return 'edit_%s.html' % group[:-1]
        return 'edit.html'


class ReturnActionMenuViewlet(ActionMenuViewlet):

    def groups(self, group_id):
        result = ''
        if group_id == 'students':
            result = _('Students')
        elif group_id == 'teachers':
            result = _('Teachers')
        elif group_id == 'administrators':
            result = _('Administrators')
        return translate(result, context=self.request)

    def __init__(self, *args, **kw):
        super(ReturnActionMenuViewlet, self).__init__(*args, **kw)
        students = activeGroupContainer().get('students')
        # XXX: assumes person is in a single group
        group_id = None
        if students is not None and self.context in students.members:
            group_id = 'students'
        teachers = activeGroupContainer().get('teachers')
        if teachers is not None and self.context in teachers.members:
            group_id = 'teachers'
        administrators = activeGroupContainer().get('administrators')
        if administrators is not None and \
           self.context in administrators.members:
            group_id = 'administrators'
        self.link = '../%s.html' % (group_id)
        self.title = _('Return to ${group}',
                       mapping={'group': self.groups(group_id)})


class ManagePersonsViewlet(NavigationViewlet):

    group_id = None

    @property
    def link(self):
        person_container = ISchoolToolApplication(None)['persons']
        active_schoolyear = activeSchoolYear()
        if active_schoolyear is None:
            return ''
        return "%s/%s.html" % (absoluteURL(person_container, self.request),
                               self.group_id)


class ManageTeachersViewlet(ManagePersonsViewlet):

    group_id = 'teachers'


class ManageStudentsViewlet(ManagePersonsViewlet):

    group_id = 'students'


class ManageAdministratorsViewlet(ManagePersonsViewlet):

    group_id = 'administrators'


class StudentFormJSViewlet(ViewletBase):

    render = ViewPageTemplateFile('templates/student_form_js.pt')
